/*
 * ConfigurationWindow.java
 *
 * Created on 24-mrt-2010, 21:04:45
 */

package happert.UI;

import happert.Game;
import happert.GameDifficulty;
import happert.GameDifficultySettings;
import happert.GameIntelligence;
import java.util.Enumeration;
import javax.swing.JRadioButton;

public class ConfigurationWindow extends javax.swing.JFrame {
    private Game game = null;

    /** Creates new form ConfigurationWindow */
    public ConfigurationWindow() {
        initComponents();
    }

    public ConfigurationWindow( Game game ) {
        this.game = game;
        initComponents();
        preInitGameOptions();
    }

    public void preInitGameOptions() {
        if ( game.getDifficultySettings().getDifficulty() != null ) {
            switch( game.getDifficultySettings().getDifficulty() ) {
                case Easy: {
                    btnDifEasy.setSelected(true);
                    slStones.setValue(30);
                    slGranites.setValue(2);
                    slHapperSpeed.setValue(800);
                } break;

                case Normal: {
                    btnDifNormal.setSelected(true);
                    slStones.setValue(20);
                    slGranites.setValue(7);
                    slHapperSpeed.setValue(450);
                } break;

                case Hard: {
                    btnDifHard.setSelected(true);
                    slStones.setValue(15);
                    slGranites.setValue(10);
                    slHapperSpeed.setValue(200);
                } break;

                case Custom: {
                    btnCustom.setSelected(true);
                    slStones.setValue( game.getDifficultySettings().getStonePercentage() );
                    slGranites.setValue( game.getDifficultySettings().getGranitePercentage() );

                    // We have to reverse the slider so the direction is left from right which
                    // is nicer from a usabilty point of view
                    slHapperSpeed.setValue( 1000 - game.getDifficultySettings().getHapperSpeed() );
                } break;

                default: break;
            }

            switch( game.getDifficultySettings().getGameIntelligence() ) {
                case Normal: {
                    btnIntelligenceNormal.setSelected(true);
                } break;

                case Smart: {
                    btnIntelligenceSmart.setSelected(true);
                } break;

                default: break;
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btnGroupDifficulty = new javax.swing.ButtonGroup();
        btnIntelligence = new javax.swing.ButtonGroup();
        btnAccept = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        btnDifEasy = new javax.swing.JRadioButton();
        btnDifNormal = new javax.swing.JRadioButton();
        btnDifHard = new javax.swing.JRadioButton();
        slStones = new javax.swing.JSlider();
        jLabel2 = new javax.swing.JLabel();
        slGranites = new javax.swing.JSlider();
        jLabel3 = new javax.swing.JLabel();
        slHapperSpeed = new javax.swing.JSlider();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        btnCustom = new javax.swing.JRadioButton();
        jLabel11 = new javax.swing.JLabel();
        btnIntelligenceNormal = new javax.swing.JRadioButton();
        btnIntelligenceSmart = new javax.swing.JRadioButton();

        btnGroupDifficulty.add(btnDifEasy);
        btnGroupDifficulty.add(btnDifNormal);
        btnGroupDifficulty.add(btnDifHard);
        btnGroupDifficulty.add(btnCustom);

        btnIntelligence.add(btnIntelligenceNormal);
        btnIntelligence.add(btnIntelligenceSmart);

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Happer opties");

        btnAccept.setText("Toepassen");
        btnAccept.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAcceptActionPerformed(evt);
            }
        });

        btnCancel.setText("Annuleren");
        btnCancel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                btnCancelMouseReleased(evt);
            }
        });

        jLabel1.setFont(new java.awt.Font("Tahoma", 1, 18)); // NOI18N
        jLabel1.setText("Moeilijkheidsniveau");

        btnDifEasy.setText("Makkelijk");
        btnDifEasy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDifNormalActionPerformed(evt);
            }
        });

        btnDifNormal.setText("Normaal");
        btnDifNormal.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDifNormalActionPerformed(evt);
            }
        });

        btnDifHard.setText("Moeilijk");
        btnDifHard.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDifNormalActionPerformed(evt);
            }
        });

        slStones.setMaximum(40);
        slStones.setMinimum(5);
        slStones.setValue(20);
        slStones.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                slStonesPropertyChange(evt);
            }
        });

        jLabel2.setText("Percentage stenen");

        slGranites.setMaximum(45);
        slGranites.setMinimum(5);
        slGranites.setValue(20);
        slGranites.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                slStonesPropertyChange(evt);
            }
        });

        jLabel3.setText("Percentage graniet");

        slHapperSpeed.setMajorTickSpacing(50);
        slHapperSpeed.setMaximum(1000);
        slHapperSpeed.setMinimum(100);
        slHapperSpeed.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                slStonesPropertyChange(evt);
            }
        });

        jLabel4.setText("Snelheid Happer");

        jLabel5.setText("Snel");

        jLabel6.setText("Langzaam");

        jLabel7.setText("Hoog");

        jLabel8.setText("Hoog");

        jLabel9.setText("Laag");

        jLabel10.setText("Laag");

        btnCustom.setText("Custom");
        btnCustom.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDifNormalActionPerformed(evt);
            }
        });

        jLabel11.setFont(new java.awt.Font("Tahoma", 1, 18)); // NOI18N
        jLabel11.setText("Happer Intelligentie");

        btnIntelligenceNormal.setText("Normaal");

        btnIntelligenceSmart.setText("Slim");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(btnIntelligenceSmart)
                    .addComponent(btnIntelligenceNormal)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jLabel1)
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(layout.createSequentialGroup()
                                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(btnDifHard)
                                        .addComponent(btnDifEasy)
                                        .addComponent(btnDifNormal))
                                    .addGap(101, 101, 101)
                                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(jLabel9)
                                        .addComponent(jLabel10)
                                        .addComponent(jLabel6)))
                                .addComponent(btnCustom))
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(layout.createSequentialGroup()
                                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                    .addComponent(jLabel3)
                                                    .addComponent(slGranites, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                    .addComponent(slHapperSpeed, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                    .addComponent(jLabel4))
                                                .addComponent(slStones, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED))
                                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                            .addComponent(btnAccept)
                                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                            .addComponent(btnCancel)
                                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)))
                                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(jLabel8)
                                        .addComponent(jLabel7)
                                        .addComponent(jLabel5)))
                                .addComponent(jLabel2))
                            .addGap(56, 56, 56)))
                    .addComponent(jLabel11))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jLabel1)
                        .addGap(18, 18, 18)
                        .addComponent(btnDifEasy)
                        .addGap(18, 18, 18)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(btnDifNormal)
                            .addComponent(jLabel9))
                        .addGap(18, 18, 18)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(btnDifHard)
                            .addGroup(layout.createSequentialGroup()
                                .addGap(19, 19, 19)
                                .addComponent(jLabel10)))
                        .addGap(18, 18, 18)
                        .addComponent(btnCustom, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(73, 73, 73)
                        .addComponent(jLabel2)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel8)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(slStones, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(18, 18, 18)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel3)
                                    .addGroup(layout.createSequentialGroup()
                                        .addGap(19, 19, 19)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addComponent(jLabel7)
                                            .addComponent(slGranites, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                                .addGap(35, 35, 35)
                                .addComponent(jLabel4)))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel6)
                    .addComponent(slHapperSpeed, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel5))
                .addGap(31, 31, 31)
                .addComponent(jLabel11)
                .addGap(18, 18, 18)
                .addComponent(btnIntelligenceNormal)
                .addGap(18, 18, 18)
                .addComponent(btnIntelligenceSmart)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 25, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnAccept)
                    .addComponent(btnCancel))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnDifNormalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDifNormalActionPerformed
        if( evt.getActionCommand().equalsIgnoreCase("Makkelijk") ) {
            slStones.setValue(30);
            slGranites.setValue(2);
            slHapperSpeed.setValue(800);
        } else if( evt.getActionCommand().equalsIgnoreCase("Normaal") ) {
            slStones.setValue(20);
            slGranites.setValue(7);
            slHapperSpeed.setValue(450);
        } else if( evt.getActionCommand().equalsIgnoreCase("Moeilijk") ) {
            slStones.setValue(15);
            slGranites.setValue(10);
            slHapperSpeed.setValue(200);
        }  else if( evt.getActionCommand().equalsIgnoreCase("Custom") ) {
            // no need to do anything
        } else {
            // geen juiste keuze
        }
    }//GEN-LAST:event_btnDifNormalActionPerformed

    private void slStonesPropertyChange(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_slStonesPropertyChange
        btnCustom.setSelected(true);
    }//GEN-LAST:event_slStonesPropertyChange

    private void btnCancelMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_btnCancelMouseReleased
        this.dispose();
    }//GEN-LAST:event_btnCancelMouseReleased

    private void btnAcceptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAcceptActionPerformed
        GameDifficultySettings newGameDifficulty = new GameDifficultySettings();

        // set the right difficulty
        for (Enumeration e=btnGroupDifficulty.getElements(); e.hasMoreElements(); ) {
            JRadioButton b = (JRadioButton)e.nextElement();
            if (b.getModel() == btnGroupDifficulty.getSelection() ) {
                if( b.getActionCommand().equalsIgnoreCase("Makkelijk") ) {
                    newGameDifficulty.setDifficulty(GameDifficulty.Easy);
                } else if( b.getActionCommand().equalsIgnoreCase("Normaal") ) {
                    newGameDifficulty.setDifficulty(GameDifficulty.Normal);
                } else if( b.getActionCommand().equalsIgnoreCase("Moeilijk") ) {
                    newGameDifficulty.setDifficulty(GameDifficulty.Hard);
                }  else if( b.getActionCommand().equalsIgnoreCase("Custom") ) {
                    newGameDifficulty.setDifficulty(GameDifficulty.Custom);
                } else {
                    // nothing to do here
                }
            }
        }

        // set percentages
        newGameDifficulty.setStonePercentage( slStones.getValue() );
        newGameDifficulty.setGranitePercentage( slGranites.getValue() );

        // We have to reverse the slider so the direction is left from right which
        // is nicer from a usabilty point of view
        newGameDifficulty.setHapperSpeed( 1000 - slHapperSpeed.getValue() );

        // set intelligence

        //normal selected
        if( btnIntelligenceNormal.isSelected() ) {
            newGameDifficulty.setGameIntelligence(GameIntelligence.Normal);
        }

        // smart selected
        if( btnIntelligenceSmart.isSelected() ) {
            newGameDifficulty.setGameIntelligence(GameIntelligence.Smart);
        }

        game.setDifficultySettings(newGameDifficulty);
        this.dispose();
    }//GEN-LAST:event_btnAcceptActionPerformed

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new ConfigurationWindow().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAccept;
    private javax.swing.JButton btnCancel;
    private javax.swing.JRadioButton btnCustom;
    private javax.swing.JRadioButton btnDifEasy;
    private javax.swing.JRadioButton btnDifHard;
    private javax.swing.JRadioButton btnDifNormal;
    private javax.swing.ButtonGroup btnGroupDifficulty;
    private javax.swing.ButtonGroup btnIntelligence;
    private javax.swing.JRadioButton btnIntelligenceNormal;
    private javax.swing.JRadioButton btnIntelligenceSmart;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JSlider slGranites;
    private javax.swing.JSlider slHapperSpeed;
    private javax.swing.JSlider slStones;
    // End of variables declaration//GEN-END:variables

}
