package happert.UI;

import happert.gameobjects.*;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

public class GameObjectRenderer {
    private static Image rockImage = null;
    private static Image graniteImage = null;
    private static Image playerImage = null;
    private static Image happertImage = null;


    private final static String BASEPATH = ".\\src\\happert\\UI\\images\\";
    
    /**
     * Render the image for a game object
     * @param obj The game object we wish to render
     * @return Image The representing image for this object
     */
    static public Image renderImage( GameObject obj ) {       
        
        if( obj instanceof Rock ) {
            if( rockImage == null) {
                rockImage = readImage(BASEPATH + "rock.jpg");
            }

            return rockImage;
        }

        if( obj instanceof Granite ) {
            if( graniteImage == null ) {
                graniteImage = readImage(BASEPATH + "granite.jpg");
            }

            return graniteImage;
        }

        if( obj instanceof Player ) {
            if (playerImage == null) {
                playerImage = readImage(BASEPATH + "player.jpg");
            }
            return playerImage;
        }

        if( obj instanceof Happert ) {
            if (happertImage == null) {
                happertImage = readImage(BASEPATH + "happert.jpg");
            }

            return happertImage;
        }

        throw new RuntimeException("Unknown GameObject, no image to render.");
    }

    /**
     * Read an image of the file system
     * @param imgLocation The image location
     * @return Image the image
     */
    static public Image readImage( String imgLocation ) {
        try {
            BufferedImage image = ImageIO.read(new File(imgLocation));
            return image;
        } catch (IOException ex) {
            throw new RuntimeException(ex.getMessage());
        }
    }
}
