package happert.UI;
import happert.playfield.Cell;
import happert.Game;
import happert.playfield.Playfield;
import java.awt.*;
import javax.swing.JOptionPane;

public class HapperPanel extends javax.swing.JPanel implements IRender {

    private int horSize, vertSize;  // de berekende grootte in punten

    private final int PIXELS_PER_CELL   = 25;

    private Game game = null;
    private KeyStateTracker keyState = new KeyStateTracker();

    public HapperPanel () {
        setLayout( null );
        setBackground( java.awt.Color.white );
        setFocusable(true);
        requestFocusInWindow();

        addKeyListener(keyState);
    }
    
    public void setGame(Game game) {
        this.game = game;

        game.attachKeyStateTracker(keyState);
    }

    /**
     * Used to paint the component on our own specific way.
     * @param g the graphics object
     */
    @Override
    public void paintComponent( Graphics g ) {
        super.paintComponent( g );              //  vereist Swing

        if (game==null || game.getPlayfield() == null) return;
        
        horSize = game.getPlayfield().getWidth() * PIXELS_PER_CELL;
        vertSize = game.getPlayfield().getHeight() * PIXELS_PER_CELL;

        setPreferredSize( new Dimension(horSize, vertSize));
        //draw grid
        drawGrid( g );

        // in Netbeans designer there won't be a game instance, exit early
        
        // iterate thru playingfield cells
        drawGameElements( g );
      
    }

    /**
     * Draw the grid on the frame
     * @param g The graphics component
     */
    private void drawGrid( Graphics g ) {
        int x = PIXELS_PER_CELL;
        int y = PIXELS_PER_CELL;

        while ( x <=  horSize  ) {
            g.drawLine(x, 0, x, vertSize );
            x += 25;
        }

        while ( y <= vertSize ) {
            g.drawLine(0, y, horSize, y );
            y += 25;
        }
    }

    /**
     * Walk through all fields and when fields has content draw
     * it's image on the playfield
     * @param g The graphics element
     */
    private void drawGameElements( Graphics g ) {
        Playfield field = game.getPlayfield();
        int horFields = field.getWidth();
        int vertFields = field.getHeight();

        for( int x = 0; x < horFields; x++ ) {
            for ( int y = 0; y < vertFields; y++ ) {
                Cell fld = field.getCell(x, y);
                if( fld != null ) {
                    if( fld.getContents() != null ) {
                        g.drawImage( GameObjectRenderer.renderImage( fld.getContents() ), ( x * PIXELS_PER_CELL ) + 1, ( y * PIXELS_PER_CELL ) + 1, this );
                    }
                }
            }
        }
    }

    /**
     * @return the keyState
     */
    KeyStateTracker getKeyState() {
        return keyState;
    }

    public void Render() {
        this.repaint();
    }

    public int viewableCellWidth() {
        return this.getWidth() / PIXELS_PER_CELL;
    }

    public int viewableCellHeight() {
        return this.getHeight() / PIXELS_PER_CELL;
    }

    public void sendMessageToWindow( HapperMessages msg ) {
        switch( msg ) {
            case WON: {
                JOptionPane.showMessageDialog(this, "Je hebt het spel gewonnen!");
            }
            break;
            case LOOSE: {
                JOptionPane.showMessageDialog(this, "Je hebt het spel verloren!");
            }
            break;
        }
    }
}
