package happert.UI;

import happert.gameobjects.DirectionEnum;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

/**
 * Een kleine 'state-machine' om bij te houden wat de status is van de
 * cursor keys.
 * We willen niet direct reageren als een toets aangeraakt wordt maar willen
 * zelf kunnen timen.
 * Object houdt daarom een mutable key state bij. De lastKey wordt gereset na uitlezen.
 *
 */
public class KeyStateTracker implements KeyListener {

    private boolean up;
    private boolean down;
    private boolean left;
    private boolean right;
    private DirectionEnum lastKey = DirectionEnum.None;

    public void keyTyped(KeyEvent e) {
        // nop
    }

    public void keyPressed(KeyEvent e) {
        switch (e.getKeyCode()) {
            case KeyEvent.VK_UP:
                up = true;
                lastKey = DirectionEnum.Up;
                break;
            case KeyEvent.VK_DOWN:
                down = true;
                lastKey = DirectionEnum.Down;
                break;
            case KeyEvent.VK_LEFT:
                left = true;
                lastKey = DirectionEnum.Left;
                break;
            case KeyEvent.VK_RIGHT:
                right = true;
                lastKey = DirectionEnum.Right;
                break;
            default:
                return;
        }
    }

    public void keyReleased(KeyEvent e) {
        switch (e.getKeyCode()) {
            case KeyEvent.VK_UP:
                up = false;
                if (lastKey == DirectionEnum.Up) {
                    lastKey = DirectionEnum.None;
                }
                break;
            case KeyEvent.VK_DOWN:
                down = false;
                if (lastKey == DirectionEnum.Down) {
                    lastKey = DirectionEnum.None;
                }
                break;
            case KeyEvent.VK_LEFT:
                left = false;
                if (lastKey == DirectionEnum.Left) {
                    lastKey = DirectionEnum.None;
                }
                break;
            case KeyEvent.VK_RIGHT:
                right = false;
                if (lastKey == DirectionEnum.Right) {
                    lastKey = DirectionEnum.None;
                }
                break;
            default:
                return;
        }
    }

    public DirectionEnum getDirection() {
        DirectionEnum dir = lastKey;
        if (dir == DirectionEnum.None) {
            // check if any key is held
            if (left) {
                return DirectionEnum.Left;
            }
            if (right) {
                return DirectionEnum.Right;
            }
            if (up) {
                return DirectionEnum.Up;
            }
            if (down) {
                return DirectionEnum.Down;
            }
        }
        
        return dir;
    }
}
