
package happert.gameobjects;

import happert.Game;
import happert.GameIntelligence;
import happert.gameobjects.happertstrategies.*;
import happert.playfield.Playfield;
import happert.playfield.Point;

public class Happert extends MovableGameObject {
    private MoveStrategy strategy;
    private Game         game;

    public Happert(Point startLocation, Playfield playfield) {
        super(startLocation, playfield);
     }

    public void setGameInstance( Game game ) {
        this.game = game;
    }

    public boolean move() {

        DirectionEnum dir = strategy.findMoveDirection();
        System.out.println( dir );
        if (dir == DirectionEnum.None) {
            return false;
        }
        
        boolean moveSuccess = this.move(dir);
        assert moveSuccess : "Moving Happert failed where it shouldn't! " + dir;

        return moveSuccess;
    }

    @Override
    public boolean pushable() {
        return false;
    }

    @Override
    protected boolean resolveBlock(GameObject targetCellContent, DirectionEnum direction) {
        // if player YUMMIE!
        if (targetCellContent instanceof Player) {
            return true;
        }
        // nop, the happert will not push obstacles
        return false;
    }

    public void huntFor(MovableGameObject prey) {
        //this.strategy = new WanderingMoveStrategy(this);
        //this.strategy = new CloseInMoveStrategy(this, prey);

        if( game != null ) {
            if( game.getDifficultySettings().getGameIntelligence() == GameIntelligence.Normal ) {
                this.strategy = new CloseInMoveStrategy(this, prey);
            } else if ( game.getDifficultySettings().getGameIntelligence() == GameIntelligence.Smart ) {
                this.strategy = new DijkstraMoveStrategy(this, prey, getPlayfield());
            }
        } else { // default strategy
            this.strategy = new CloseInMoveStrategy(this, prey);
        }
    }

}
