package happert.gameobjects.happertstrategies;

import happert.gameobjects.*;
import happert.playfield.*;

public class CloseInMoveStrategy extends MoveStrategy {

    boolean flip = false;

    public CloseInMoveStrategy(MovableGameObject host, MovableGameObject goal) {
        super(host, goal);
    }

    public DirectionEnum findMoveDirection() {
        assert getGoal() != null : "Goal must be set for this strategy";

        // find on which axis the distance to goal is longest
        DirectionEnum largeDir = calcDirectionForLongestAxisDistance();
        if (largeDir == DirectionEnum.None) {
            // we are already ON the goal
            // TODO exception worthy?
            return largeDir;
        }
        // try move towards goal on that axis
        if (canMove(largeDir)) {
            return largeDir;
        }

        // if blocked try the other axis
        DirectionEnum shortDir = calcDirectionForShortestAxisDistance();
        if (shortDir == DirectionEnum.None) {
            // aligned on the longest axis
            // only option is to deviate from it
            // flip a coin
            boolean polarity = (Math.random() < 0.5d);

            DirectionEnum devDir = calcDirectionDeviatingFromAlignedAxis(polarity);
            if (canMove(devDir)) {
                return devDir;
            } else {
                devDir = devDir.inverse();
                if (canMove(devDir)) {
                    return devDir;
                }
            }

        } else {
            if (canMove(shortDir)) {
                return shortDir;
            }
            // if blocked try moving opposite of the short axis
            shortDir = shortDir.inverse();
            if (canMove(shortDir)) {
                return shortDir;
            }
        }

        // if blocked try moving opposite of the long axis
        largeDir = largeDir.inverse();
        if (canMove(largeDir)) {
            return largeDir;
        }

        return DirectionEnum.None;
    }

    private DirectionEnum calcDirectionForLongestAxisDistance() {
        Point moveVector = getGoal().getLocation()
                                .minus(getHost().getLocation());

        if (moveVector.equals(new Point(0,0))) {
            return DirectionEnum.None;
        }
        if (Math.abs(moveVector.X()) > Math.abs(moveVector.Y())) {
            // move along X axis
            return calcDirectionForX(moveVector);
        } else {
            // move along Y axis
            return calcDirectionForY(moveVector);
        }

    }

    private DirectionEnum calcDirectionForShortestAxisDistance() {
        Point moveVector = getGoal().getLocation()
                                .minus(getHost().getLocation());

        if (moveVector.equals(new Point(0,0))) {
            return DirectionEnum.None;
        }
        if (Math.abs(moveVector.X()) <= Math.abs(moveVector.Y())) {
            // move along X axis
            return calcDirectionForX(moveVector);
        } else {
            // move along Y axis
            return calcDirectionForY(moveVector);
        }

    }

    private DirectionEnum calcDirectionForY(Point moveVector) {
        if (moveVector.Y() == 0) {
            return DirectionEnum.None;
        }
        if (moveVector.Y() < 0) {
            return DirectionEnum.Up;
        } else {
            return DirectionEnum.Down;
        }
    }

    private DirectionEnum calcDirectionForX(Point moveVector) {
        if (moveVector.X() == 0) {
            return DirectionEnum.None;
        }
        if (moveVector.X() < 0) {
            return DirectionEnum.Left;
        } else {
            return DirectionEnum.Right;
        }
    }

    private DirectionEnum calcDirectionDeviatingFromAlignedAxis(boolean polarity) {
        Point moveVector = getGoal().getLocation().minus(getHost().getLocation());
        if (moveVector.X() == 0) {
            if (polarity) {
                return DirectionEnum.Left;
            } else {
                return DirectionEnum.Right;
            }

        } else {
           if (polarity) {
                return DirectionEnum.Up;
            } else {
                return DirectionEnum.Down;
            }
        }

    }

}
