package happert.gameobjects.happertstrategies;

import happert.gameobjects.*;
import happert.playfield.Point;

public abstract class MoveStrategy {
    private MovableGameObject host;
    private GameObject goal;

    public abstract DirectionEnum findMoveDirection();

    public MoveStrategy(MovableGameObject host, MovableGameObject goal) {
        assert host != null : "Host shouldn't be null";
        this.host = host;
        this.goal = goal;
    }
    
    protected boolean canMove(DirectionEnum direction) {
        assert direction != DirectionEnum.None : "direction is set to none";
        Point targetLoc = host.calculateNewLocation(direction);
        if (targetLoc == null) {
            return false;
        }
        GameObject target = host.getPlayfield().getCellContents(targetLoc);
        return target == null || target instanceof Player;
    }

    /**
     * @return the host
     */
    public MovableGameObject getHost() {
        return host;
    }

    /**
     * @return the goal
     */
    protected GameObject getGoal() {
        return goal;
    }

}
