package happert.gameobjects.happertstrategies;

import happert.Util;
import happert.gameobjects.*;

import java.util.ArrayList;
import java.util.List;

public class WanderingMoveStrategy extends MoveStrategy {
    private DirectionEnum previousDir = DirectionEnum.None;

    public WanderingMoveStrategy(MovableGameObject host) {
        super(host, null);
    }

    public DirectionEnum findMoveDirection() {
        List<DirectionEnum> possibleMoves = findPossibleMoves();
        if (possibleMoves.size() == 0) {
            return DirectionEnum.None;
        }

        // if we have more then 1 possible move see if the inverse of our previous
        // move is in the list; if so remove it.
        if (possibleMoves.size() > 1 && previousDir != DirectionEnum.None) {
            DirectionEnum rev = previousDir.inverse();
            if (possibleMoves.contains(rev)) {
                possibleMoves.remove(rev);
            }
        }

        DirectionEnum result = pickRandomDirectionFromList(possibleMoves);
        previousDir = result;
        return result;
    }

    private DirectionEnum pickRandomDirectionFromList(List<DirectionEnum> possibleMoves) {
        DirectionEnum dir;
        // pick one randomly
        int index = Util.randomNumber(0, possibleMoves.size() - 1);
        dir = possibleMoves.get(index);
        return dir;
    }

    private List<DirectionEnum> findPossibleMoves() {
        // figure out a direction to move in
        // compile a list of possible moves
        List<DirectionEnum> possibleMoves = new ArrayList<DirectionEnum>();
        if (canMove(DirectionEnum.Left)) {
            possibleMoves.add(DirectionEnum.Left);
        }
        if (canMove(DirectionEnum.Right)) {
            possibleMoves.add(DirectionEnum.Right);
        }
        if (canMove(DirectionEnum.Up)) {
            possibleMoves.add(DirectionEnum.Up);
        }
        if (canMove(DirectionEnum.Down)) {
            possibleMoves.add(DirectionEnum.Down);
        }
        return possibleMoves;
    }
}
