package happert.playfield;

import happert.gameobjects.GameObject;

public class Playfield {

    private Cell[][] playfield;
    private int width;
    private int height;

    public Playfield(int width, int height) {
        assert width > 0 : "width needs to be larger then zero";
        assert height > 0 : "height needs to larger then zero";

        playfield = new Cell[width][height];
        this.width = width;
        this.height = height;
    }

    public Cell getCell(int x, int y) {
        assert x >= 0 : "x need to be zero of positive";
        assert x < width : "x is out of bound";
        assert y >= 0 : "y need to be zero of positive";
        assert y < height : "y is out of bound";

        return playfield[x][y];
    }

    public Cell getCell(Point point) {
        assert point.X() < width : "x is out of bound";
        assert point.Y() < height : "y is out of bound";

        return playfield[point.X()][point.Y()];
    }

    
    public void setCell(int x, int y, Cell cell) {
        assert x >= 0 : "x need to be zero of positive";
        assert x < width : "x is out of bound";
        assert y >= 0 : "y need to be zero of positive";
        assert y < height : "y is out of bound";

        playfield[x][y] = cell;
    }

    public void setCell(Point point, Cell cell) {
        assert point.X() < width  : "x is out of bound";
        assert point.Y() < height : "y is out of bound";

        playfield[point.X()][point.Y()] = cell;
    }

    public void setCellContents(Point point, GameObject gameEntity) {
        if (getCell(point) == null) {
            setCell(point, new Cell());
        }
        getCell(point).setContents(gameEntity);
    }

    public GameObject getCellContents(Point point) {
        assert point != null : "Point is null!!";
        if (getCell(point) == null) {
            setCell(point, new Cell());
        }
        return getCell(point).getContents();
    }

    /**
     * @return the width
     */
    public int getWidth() {
        return width;
    }

    /**
     * @return the height
     */
    public int getHeight() {
        return height;
    }
    
}
