package happert.playfield;

import happert.Game;
import happert.gameobjects.*;
import happert.Util;

public abstract class PlayfieldBuilder {

    protected GameObjectBuilder goBuilder;
    protected Happert happert;
    protected Player player;
    protected Playfield playfield;

    public abstract void construct(Game game);

    /**
     *
     * @param width 
     * @param height
     * @return A random position within the dimension of the playfield
     */
    protected static Point createRandomPoint(int width, int height) {
        return new Point(Util.randomNumber(0, width - 1), Util.randomNumber(0, height - 1));
    }

    /**
     * Find an empty cell and put a game object on it
     *
     * @param gameObject The object type
     * @param number Number of objects to be placed on the field
     */
    protected void distributeGameObject(GameObjectBuilder.GameObjectEnum gameObject, int number) {
        for (int i = 0; i < number; i++) {
            Point rockPos = findEmptyCell();
            playfield.setCellContents(rockPos, goBuilder.createGameObject(gameObject, rockPos));
        }
    }

    /**
     * Find an empty cell on the playfield
     * @return Point An empty position on the playfield
     */
    protected Point findEmptyCell() {
        Point pos;
        // iterate untill finding empty cell
        do {
            pos = createRandomPoint(playfield.getWidth(), playfield.getHeight());
        } while (playfield.getCellContents(pos) != null);
        return pos;
    }

    /**
     * @return the positioned happert
     */
    public Happert getHappert() {
        return happert;
    }

    /**
     * @return the positioned player
     */
    public Player getPlayer() {
        return player;
    }

    /**
     *
     * @return The constructed playfield
     */
    public Playfield getPlayfield() {
        return this.playfield;
    }

}
