package happert.playfield;

import happert.gameobjects.DirectionEnum;

public class Point {
    private int x;
    private int y;

    public Point() {
        x = 0;
        y = 0;
    }

    public Point(int X, int Y) {
        x = X;
        y = Y;
    }

    /**
     * @return the x
     */
    public int X() {
        return x;
    }

    /**
     * @param x the x to set
     */
    public void setX(int x) {
        this.x = x;
    }

    /**
     * @return the y
     */
    public int Y() {
        return y;
    }

    /**
     * @param y the y to set
     */
    public void setY(int y) {
        this.y = y;
    }

    // volgens spec, hashCode moet redelijk unique zijn en moet samengesteld
    // worden uit dezelfde velden als de equals override.
    @Override
    public int hashCode() {
        int hash = 5;
        hash = 47 * hash + this.x;
        hash = 47 * hash + this.y;
        return hash;
    }

    @Override
    public boolean equals(Object aThat) {
        if (aThat == this) return true;
        if (!(aThat instanceof Point)) return false;

        Point that = (Point)aThat;
        if (this.x == that.X()) {
            if (this.y == that.Y()) return true;
        }
        return false;
   }

    @Override
    public String toString() {
        return "(" + this.x + ", " + this.y + ")";
    }

    public int deltaDistance(Point b) {
        return Math.abs(this.x - b.x) +
                Math.abs(this.y - b.y);

    }

    public Point minus(Point b) {
        return new Point(this.x - b.x, this.y - b.y);
    }

    
    /**
     * Calculates a new point moving in a given direction from the current point.
     * Does a boundary check to see if the new point is valid.
     * Directions are calculated against a top-left coordinate system.
     *
     * @param direction the direction to move in
     * @param maxWidth Maximum width to define the boundary
     * @param maxHeight Maximum height to define the boundary
     * @return The calculated point or null if moving in the given direction was not possible
     */
    public Point moveDirection(DirectionEnum direction, int maxWidth, int maxHeight) {
        if (direction == null) throw new IllegalArgumentException("Direction argument is null.");

        
        switch (direction) {
            case None:
                assert direction != DirectionEnum.None : "Direction is set to none; doesn't lead to a new location.";
                break;
            case Left:
                if (this.X() > 0) {
                    return new Point(this.X() - 1, this.Y());
                }
                break;
            case Right:
                if (this.X() + 1 < maxWidth) {
                    return new Point(this.X() + 1, this.Y());
                }
                break;
            case Up:
                if (this.Y() > 0) {
                    return new Point(this.X(), this.Y() - 1);
                }
                break;
            case Down:
                if (this.Y() + 1 < maxHeight) {
                    return new Point(this.X(), this.Y() + 1);
                }
                break;
        }
        // we return null if moving is not valid
        return null;
    }

}
