package happert.playfield;

import happert.Game;
import happert.gameobjects.GameObjectBuilder;
import happert.gameobjects.Happert;
import happert.gameobjects.Player;

public class RandomPlayfieldBuilder extends PlayfieldBuilder {

    private int PERCENTAGE_OF_ROCKS = 20;
    private int PERCENTAGE_OF_GRANITE = 10;
    
    private final static int DEFAULT_WIDTH = 15;
    private final static int DEFAULT_HEIGHT = 20;
    private int width = DEFAULT_WIDTH;
    private int height = DEFAULT_HEIGHT;

    @Override
    public void construct(Game game) {
        playfield = new Playfield(width, height);
        goBuilder = new GameObjectBuilder(playfield);

        // set game settings
        PERCENTAGE_OF_ROCKS   = game.getDifficultySettings().getStonePercentage();
        PERCENTAGE_OF_GRANITE = game.getDifficultySettings().getGranitePercentage();

        final int cellCountPerc = (width * height)/100;

        // place rocks
        distributeGameObject(GameObjectBuilder.GameObjectEnum.ROCK, cellCountPerc * PERCENTAGE_OF_ROCKS);

        // place granite
        distributeGameObject(GameObjectBuilder.GameObjectEnum.GRANITE, cellCountPerc * PERCENTAGE_OF_GRANITE);

        // find a cell to have the player start in
        player = (Player)goBuilder.createGameObject(
                    GameObjectBuilder.GameObjectEnum.PLAYER,
                    findEmptyCell()
                );
        playfield.setCellContents(player.getLocation(), player);

        // Search an empty spot for the happert
        // make sure location is not on top of player
        Point happertLocation;
        do {
            happertLocation = findEmptyCell();
        } while (happertLocation.deltaDistance(player.getLocation()) < 6);

        happert = (Happert)goBuilder.createGameObject(
                    GameObjectBuilder.GameObjectEnum.HAPPERT,
                    happertLocation
                );

        // set reference to the game instance
        happert.setGameInstance(game);
        
        playfield.setCellContents(happertLocation, happert);
    }

    public void setCellDimensions(int viewableCellWidth, int viewableCellHeight) {
        assert viewableCellWidth > 0 : "cell width must be larger then zero";
        assert viewableCellHeight > 0 : "cell height must be larger then zero";

        this.width = viewableCellWidth;
        this.height = viewableCellHeight;
    }



}