package happert;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

public class GameTimeTrackerTest {
    private class StubbedTimeTracker extends Game.TimeTracker {
        private long currentTime;

        public StubbedTimeTracker(long frequency) {
            super(frequency);
        }

        public void setCurrentTime(long currentTime) {
            this.currentTime = currentTime;
        }

        @Override
        protected long getCurrentTime() {
            return currentTime;
        }
    }
    
    public GameTimeTrackerTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
        
    }

    @After
    public void tearDown() {
    }

    @Test
    public void EnoughTimePassed_PulseCheck_ShouldPass() {
        // arrange
        long pulse = 100;
        long startTime = 20;
        long checkTime = 130;
        StubbedTimeTracker timeTracker = new StubbedTimeTracker(pulse); // pulse frequency
        timeTracker.setCurrentTime(startTime);
        // act
        timeTracker.startTiming();

        timeTracker.setCurrentTime(checkTime); // fake time
        // assert
        boolean timeForPulse = timeTracker.isPulseTime();
        assertTrue(timeForPulse);
    }

    @Test
    public void NotEnoughTimePassed_PulseCheck_ShouldNotPass() {
        // arrange
        long pulse = 100;
        long startTime = 20;
        long checkTime = 110;

        StubbedTimeTracker timeTracker = new StubbedTimeTracker(pulse); // pulse frequency
        timeTracker.setCurrentTime(startTime);
        // act
        timeTracker.startTiming();

        timeTracker.setCurrentTime(checkTime); // fake time
        // assert
        boolean timeForPulse = timeTracker.isPulseTime();
        assertFalse(timeForPulse);
    }

    @Test
    public void EnoughTimePassed_NextPulseTime_ShouldBeNegative() {
        // arrange
        long pulse = 100;
        long startTime = 20;
        long checkTime = 140; // should be 20 past pulse

        StubbedTimeTracker timeTracker = new StubbedTimeTracker(pulse); // pulse frequency
        timeTracker.setCurrentTime(startTime);
        // act
        timeTracker.startTiming();

        timeTracker.setCurrentTime(checkTime); // fake time
        // assert
        long timeTillPulse = timeTracker.timeTillNextPulse();
        assertTrue(timeTillPulse < 0);
    }

    @Test
    public void NotEnoughTimePassed_NextPulseTime_ShouldBePositive() {
        // arrange
        long pulse = 100;
        long startTime = 20;
        long checkTime = 110; // should be 10 before pulse

        StubbedTimeTracker timeTracker = new StubbedTimeTracker(pulse); // pulse frequency
        timeTracker.setCurrentTime(startTime);
        // act
        timeTracker.startTiming();

        timeTracker.setCurrentTime(checkTime); // fake time
        // assert
        long timeTillPulse = timeTracker.timeTillNextPulse();
        assertTrue(timeTillPulse > 0);
    }

    @Test
    public void NotEnoughTimePassed_NextPulseTime() {
        // arrange
        long pulse = 100;
        long startTime = 20;
        long checkTime = 110; // should be 10 before pulse
        long expected = 10; // checktime - pulse

        StubbedTimeTracker timeTracker = new StubbedTimeTracker(pulse); // pulse frequency
        timeTracker.setCurrentTime(startTime);
        // act
        timeTracker.startTiming();

        timeTracker.setCurrentTime(checkTime); // fake time
        // assert
        long timeTillPulse = timeTracker.timeTillNextPulse();
        assertEquals(expected, timeTillPulse);
    }

    @Test
    public void PulseCompleted_NextPulseTime_ShouldEqualPulseFrequency() {
        // arrange
        long pulse = 100;
        long startTime = 10;
        long checkTime = 140; // should be 10 before pulse
        long expected = 100; // = pulse

        StubbedTimeTracker timeTracker = new StubbedTimeTracker(pulse); // pulse frequency
        timeTracker.setCurrentTime(startTime);
        // act
        timeTracker.startTiming();
        timeTracker.setCurrentTime(checkTime); // fake time
        timeTracker.pulseCompleted();

        // assert
        long timeTillPulse = timeTracker.timeTillNextPulse();
        assertEquals(expected, timeTillPulse);
    }
    
    @Test
    public void CalculateTimeGap() {
        Game g = new Game();
        assertEquals( 125, g.calculateTimeGap(250, 125));
        assertEquals( 125, g.calculateTimeGap(125, 250));
    }
}