
package happert.gameobjects.happertstrategies;

import happert.gameobjects.*;
import happert.playfield.PlayfieldBuilder;
import happert.testutil.TestUtility;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

public class CloseInMoveStrategyTest {

    public CloseInMoveStrategyTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of findMoveDirection method, of class CloseInMoveStrategy.
     */
    @Test
    public void UnblockedPathNorth_findMoveDirection_ShouldReturnUp() {
        // arrange
        String [] pft = {   ".P.",
                            "...",
                            ".H.",
                            "..."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Up, result);
    }

    @Test
    public void UnblockedPathEast_findMoveDirection_ShouldReturnRight() {
        // arrange
        String [] pft = {   ".....",
                            ".H..P",
                            "....."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Right, result);
    }

    @Test
    public void BlockedPathNorth_findMoveDirection_ShouldReturnLeftOrRight() {
        // arrange
        String [] pft = {   "..P..",
                            "..#..",
                            "..H..",
                            "....."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertTrue(result == DirectionEnum.Left
                    || result == DirectionEnum.Right);
    }

    @Test
    public void BlockedPathEast_findMoveDirection_ShouldReturnUpOrDown() {
        // arrange
        String [] pft = {   "....",
                            ".H#P",
                            "....",
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertTrue(result == DirectionEnum.Up
                    || result == DirectionEnum.Down);
    }

    @Test
    public void VerticalDistanceIsLargest_findMoveDirection_ShouldReturnUp() {
        // arrange
        String [] pft = {   "..P..",
                            ".....",
                            ".....",
                            ".H...",
                            "....."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Up, result);
    }

    @Test
    public void HorizontalDistanceIsLargest_findMoveDirection_ShouldReturnRight() {
        // arrange
        String [] pft = {   ".....",
                            ".H...",
                            ".....",
                            "....P",
                            "....."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Right, result);
    }

    @Test
    public void VerticalDistanceIsLargestButBlocked_findMoveDirection_ShouldReturnRight() {
        // arrange
        String [] pft = {   "..P..",
                            ".....",
                            ".#...",
                            ".H...",
                            "....."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Right, result);
    }

    @Test
    public void HorizontalDistanceIsLargestButBlocked_findMoveDirection_ShouldReturnDown() {
        // arrange
        String [] pft = {   ".....",
                            ".H#..",
                            ".....",
                            "....P",
                            "....."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Down, result);
    }

    @Test
    public void HorizontalDistanceIsLargestBothAxisblocked_findMoveDirection_ShouldReturnUp() {
        // arrange
        String [] pft = {   ".....",
                            ".H#..",
                            ".#...",
                            "....P",
                            "....."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Up, result);
    }

    @Test
    public void VerticalDistanceIsLargestBothAxisblocked_findMoveDirection_ShouldReturnLeft() {
        // arrange
        String [] pft = {   "..P..",
                            ".....",
                            ".#...",
                            ".H#..",
                            "....."
                        };
        CloseInMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Left, result);
    }

    private CloseInMoveStrategy setupStrategyWithStringTemplate(String[] pft) {
        PlayfieldBuilder pfb = TestUtility.constructPlayfieldFromStringTemplate(pft);
        CloseInMoveStrategy instance = new CloseInMoveStrategy(pfb.getHappert(), pfb.getPlayer());
        return instance;
    }
}
