
package happert.gameobjects.happertstrategies;

import happert.gameobjects.DirectionEnum;
import happert.playfield.PlayfieldBuilder;
import happert.testutil.TestUtility;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

public class DijkstraMoveStrategyTest {
    @BeforeClass
    public static void setUpClass() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    /**
     * Test of findMoveDirection method, of class DijkstraMoveStrategy.
     */
    @Test
    public void UnblockedPathNorth_findMoveDirection_ShouldReturnUp() {
        // arrange
        String [] pft = {   ".P.",
                            "...",
                            ".H.",
                            "..."
                        };
        DijkstraMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Up, result);
    }

    @Test
    public void UnblockedPathEast_findMoveDirection_ShouldReturnRight() {
        // arrange
        String [] pft = {   ".....",
                            ".H..P",
                            "....."
                        };
        
        DijkstraMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Right, result);
    }

    @Test
    public void BoxedIn_findMoveDirection_ShouldReturnNone() {
        // arrange
        String [] pft = {   "###..",
                            "#H#.P",
                            "###.."
                        };

        DijkstraMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.None, result);
    }

    @Test
    public void BlockedPath_findMoveDirection_ShouldBackOutToLeft() {
        // arrange
        String [] pft = {   "###..",
                            ".H.#P",
                            ".##..",
                            "....."
                        };

        DijkstraMoveStrategy instance = setupStrategyWithStringTemplate(pft);
        // act
        DirectionEnum result = instance.findMoveDirection();
        // assert
        assertEquals(DirectionEnum.Left, result);
    }

    private DijkstraMoveStrategy setupStrategyWithStringTemplate(String[] pft) {
        PlayfieldBuilder pfb = TestUtility.constructPlayfieldFromStringTemplate(pft);
        DijkstraMoveStrategy instance = new DijkstraMoveStrategy(pfb.getHappert(), pfb.getPlayer(), pfb.getPlayfield());
        
        return instance;
    }
}
