package happert.playfield;

import org.junit.Test;
import static org.junit.Assert.*;


public class PointTest {

    @Test
    public void InitPoint() {
        Point tp = new Point(10, 15);
        assertEquals(10, tp.X() );
        assertEquals(15, tp.Y() );
    }

    @Test
    public void Setters() {
        Point tp = new Point(0, 0);
        tp.setX(10);
        tp.setY(15);
        assertEquals(10, tp.X() );
        assertEquals(15, tp.Y() );
    }

    @Test
    public void ValueEquality() {
        Point a = new Point(0,0);
        Point b = new Point(0,0);
        assertEquals(a, b);
    }

    @Test
    public void NotEqual() {
        Point a = new Point(0,0);
        Point b = new Point(1,1);
        assertFalse(a.equals(b));
    }

    @Test
    public void InstanceEquality() {
        Point a = new Point(0,0);
        assertEquals(a, a);
    }

    @Test
    public void OnlyEqualsSameType() {
        Point a = new Point(0,0);
        int b = 0;
        assertFalse(a.equals(b));
    }

    @Test
    public void EqualsNullSafe() {
        Point a = new Point(0,0);
        assertFalse(a.equals(null));
    }

    @Test
    public void DeltaDistance() {
        Point a = new Point(0, 0);
        assertEquals(1, a.deltaDistance(new Point(0, 1)));
        // distance should be absolute
        assertEquals(1, a.deltaDistance(new Point(0, -1)));
        assertEquals(3, a.deltaDistance(new Point(3, 0)));
        // diagonals aren't supported so distance is always dX + dY
        assertEquals(6, new Point(6,6).deltaDistance(new Point(3,3)));
        assertEquals(0, a.deltaDistance(new Point(0,0)));
    }

    @Test
    public void Minus() {
        // arrange
        Point a = new Point(0,0);
        Point b = new Point(1,1);
        // act
        Point result = a.minus(b);
        // assert
        assertEquals(new Point(-1, -1), result);
    }

    
}