package happert.playfield;

import happert.gameobjects.DirectionEnum;
import org.junit.Test;
import static org.junit.Assert.*;

public class Point_moveDirectionTest {
    private int someWidth = 5;
    private int someHeight = 5;
    
    @Test
    public void moveDirection_Left_AtLeftEdge_ShouldNotBeAbletoMove() {
        // arrange
        Point a = new Point(0,0);
        DirectionEnum direction = DirectionEnum.Left;
        // act
        Point result = a.moveDirection(direction, someWidth, someHeight);
        //assert
        assertNull(result);
    }

    @Test
    public void moveDirection_Down_AtBottomEdge_ShouldNotBeAbletoMove() {
        // arrange
        Point a = new Point(0,5);
        int height = 5;
        DirectionEnum direction = DirectionEnum.Down;
        // act
        Point result = a.moveDirection(direction, someWidth, height);
        //assert
        assertNull(result);
    }

    @Test
    public void moveDirection_Right_AtRightEdge_ShouldNotBeAbletoMove() {
        // arrange
        Point a = new Point(5,0);
        int width = 5;
        DirectionEnum direction = DirectionEnum.Right;
        // act
        Point result = a.moveDirection(direction, width, someHeight);
        //assert
        assertNull(result);
    }

    @Test
    public void moveDirection_Up_AtTopEdge_ShouldNotBeAbletoMove() {
        // arrange
        Point a = new Point(0,0);
        DirectionEnum direction = DirectionEnum.Up;
        // act
        Point result = a.moveDirection(direction, someWidth, someHeight);
        //assert
        assertNull(result);
    }

    @Test
    public void moveDirection_Up_NotAtEdge_ShouldMove() {
        // arrange
        Point a = new Point(1,1);
        DirectionEnum direction = DirectionEnum.Up;
        // act
        Point result = a.moveDirection(direction, someWidth, someHeight);
        //assert
        assertEquals(new Point(1,0), result);
    }

    @Test
    public void moveDirection_Down_NotAtEdge_ShouldMove() {
        // arrange
        Point a = new Point(1,1);
        DirectionEnum direction = DirectionEnum.Down;
        // act
        Point result = a.moveDirection(direction, someWidth, someHeight);
        //assert
        assertEquals(new Point(1,2), result);
    }

    @Test
    public void moveDirection_Left_NotAtEdge_ShouldMove() {
        // arrange
        Point a = new Point(1,1);
        DirectionEnum direction = DirectionEnum.Left;
        // act
        Point result = a.moveDirection(direction, someWidth, someHeight);
        //assert
        assertEquals(new Point(0,1), result);
    }

    @Test
    public void moveDirection_Right_NotAtEdge_ShouldMove() {
        // arrange
        Point a = new Point(1,1);
        DirectionEnum direction = DirectionEnum.Right;
        // act
        Point result = a.moveDirection(direction, someWidth, someHeight);
        //assert
        assertEquals(new Point(2,1), result);
    }

    @Test
    public void moveDirection_NullDirection_ShouldThrowIllegalArgumentException() {
        // arrange

        DirectionEnum direction = null;
        // act
        Point somePoint = new Point(1,1);
        boolean thrown = false;
        try {
            Point aResult = somePoint.moveDirection(direction, someWidth, someHeight);
        } catch (IllegalArgumentException ex) {
            thrown = true;
        }

        //assert
        assertTrue(thrown);
    }

    @Test
    public void moveDirection_WhenAssertsEnabled_NoneDirection_ShouldAssert() {
        // arrange
        DirectionEnum direction = DirectionEnum.None;
        // act
        Point somePoint = new Point(1,1);
        boolean thrown = false;
        try {
            Point aResult = somePoint.moveDirection(direction, someWidth, someHeight);
        } catch (AssertionError err) {
            thrown = true;
        }

        //assert

        // this is a rare special case where one would use an if statement in a test
        // we don't want this test to fail when assertions are disabled
        if (areAssertsEnabled()) {
            assertTrue(thrown);
        }
    }

    @Test
    public void moveDirection_WhenAssertsDisabled_NoneDirection_ReturnsNone() {
        // arrange
        DirectionEnum direction = DirectionEnum.None;
        // act
        Point somePoint = new Point(1,1);
        boolean thrown = false;
        Point result = null;
        try {
            result = somePoint.moveDirection(direction, someWidth, someHeight);
        } catch (AssertionError err) {
            thrown = true;
        }
        // assert

        // this is a rare special case where one would use an if statement in a test
        // we don't want this test to fail when assertions are enabled
        if (!thrown) {
            assertNull(result);
        }
    }

    // this checks if the VM Option '-enabledassertions" has been set for the project or not
    private boolean areAssertsEnabled() {
        boolean assertsEnabled = false;
        assert assertsEnabled = true;
        return assertsEnabled;
    }
}
