
package happert.playfield;

import happert.Game;
import happert.gameobjects.GameObjectBuilder;
import java.util.ArrayList;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

public class RandomPlayfieldBuilderTest {
    private RandomPlayfieldBuilder instance;

    @Before
    public void setUp() {
        // we mock the distribution method
        instance = new stubPlayfieldBuilder();
        instance.construct(new Game());
    }

    @After
    public void tearDown() {
    }

    @Test
    public void ConstructShouldDistributeRocks() {
        assertTrue(((stubPlayfieldBuilder)instance).distributed.contains(GameObjectBuilder.GameObjectEnum.ROCK));
    }

    @Test
    public void ConstructShouldDistributeGranite() {
        assertTrue(((stubPlayfieldBuilder)instance).distributed.contains(GameObjectBuilder.GameObjectEnum.GRANITE));
    }

    @Test
    public void ConstructShouldCreatePlayfield() {
        Playfield pf = instance.getPlayfield();
        assertNotNull(pf);
        assertTrue(pf.getHeight() > 0);
        assertTrue(pf.getWidth() > 0);
    }

    @Test
    public void ConstructShouldSetPlayerLocation() {
        assertNotNull(instance.getPlayer().getLocation());
    }

    @Test
    public void ConstructShouldSetHapperLocation() {
        assertNotNull(instance.getHappert().getLocation());
    }

    @Test
    public void ConstructPlayerLocationDifferentThenHapperLocation() {
        assertFalse(instance.getPlayer().getLocation().equals(instance.getHappert().getLocation()));
    }

    // small mock class to check object distribution
    public class stubPlayfieldBuilder extends RandomPlayfieldBuilder {
        protected ArrayList<GameObjectBuilder.GameObjectEnum> distributed = new ArrayList<GameObjectBuilder.GameObjectEnum>();

        @Override
        public void distributeGameObject(GameObjectBuilder.GameObjectEnum gameObject, int number) {
            distributed.add(gameObject);
        }

    }

}